/*  MaViLla: Marcador y Visualizador de Llamadas
 *
 *  Copyright (C) 2011  Area de Servicios Telematicos
 *			CSIRC
 *			Universidad de Granada
 *			telematica@ugr.es
 *  
 *  Version: 1.1
 *  Last Modified: 30 May 2011
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "registro_llamadas.h"
#include "ui_registro_llamadas.h"
#include <QtGui>
#include <QDateTime>
#include <QAction>

Registro_llamadas::Registro_llamadas(Entrada * t, QString dirpath, QWidget *parent) :
    QDialog(parent),
    path(dirpath),
    ui(new Ui::Registro_llamadas)
{
    ui->setupUi(this);
    // Creamos el dialogo de llamada saliente
    llam_sal = new Llamada_saliente(this);
    ent = t;
    // Enlazamos los slots
    connect(ui->pushButton,SIGNAL(clicked()),this,SLOT(hide()));
    connect(ui->tableView,SIGNAL(doubleClicked(QModelIndex)),this,SLOT(llamar()));
    // Crear las acciones y los menus
    crearAcciones();
    crearMenusContextuales();
    // Creamos el modelo e insertamos las columnas
    modelo = new QStandardItemModel(0, 3, this);
    modelo->setHeaderData(0, Qt::Horizontal, QObject::tr("Nombre"));
    modelo->setHeaderData(1, Qt::Horizontal, QObject::tr("Telfono"));
    modelo->setHeaderData(2, Qt::Horizontal, QObject::tr("Fecha"));
    // Ponemos los tamaos
    ui->tableView->setModel(modelo);
    ui->tableView->setColumnWidth(0,223);
    ui->tableView->setColumnWidth(1,108);
    ui->tableView->setColumnWidth(2,127);
    // Establecemos la ordenacion
    ui->tableView->setSortingEnabled(true);
    ui->tableView->sortByColumn(2,Qt::DescendingOrder);
    // Quitando los botones de maximizar y minimizar
    Qt::WindowFlags flags = Qt::Window | Qt::WindowCloseButtonHint;
    this->setWindowFlags(flags);
    // Ponemos las etiquetas con colores
    QPalette plt;
    plt.setColor(QPalette::WindowText, Qt::red);
    ui->label_3->setPalette(plt);
    ui->label_6->setPalette(plt);
    plt.setColor(QPalette::WindowText, Qt::darkGreen);
    ui->label->setPalette(plt);
    ui->label_4->setPalette(plt);
    plt.setColor(QPalette::WindowText, Qt::darkBlue);
    ui->label_2->setPalette(plt);
    ui->label_5->setPalette(plt);
    // Ponemos los contadores a 0
    n_ent = n_sal = n_imp = 0;
}

/* Aade una entrada al listado */
void Registro_llamadas::addEntrada(const QString &nombre,const QString &telefono, const bool &imp, const bool &entr)
{
    // Creo una fila y la relleno
    QDateTime t = QDateTime::currentDateTime();
    QColor color = QColor(Qt::black);
    modelo->insertRow(0);
    ui->tableView->setRowHeight(0,22);
    modelo->setData(modelo->index(0, 0), nombre);
    modelo->setData(modelo->index(0, 1), telefono);
    modelo->setData(modelo->index(0, 2), t.toString("dd/MM/yyyy hh:mm:ss"));
    // Determino los colores de las llamadas y el no de ellas
    if( !entr )
    {
        color = QColor(Qt::darkBlue);
        n_sal++;
    }
    else
    {
        if( imp )
        {
            color = QColor(Qt::red);
            n_imp++;
        }
        else
        {
            color = QColor(Qt::darkGreen);
            n_ent++;
        }
    }
    ui->label_4->setText(QString::number(n_ent));
    ui->label_5->setText(QString::number(n_sal));
    ui->label_6->setText(QString::number(n_imp));
	// Coloreo la linea
    for(int i=0; i<modelo->columnCount(); ++i)
    {
     //           modelo->setData(modelo->index(0, i), QColor(Qt::red), Qt::BackgroundColorRole);
        modelo->setData(modelo->index(0, i), color, Qt::ForegroundRole);
    }

	if( imp && QDir(path).exists("ring.wav")) // Reproduzco un pequeo ring si es importante
		QSound::play(path + "ring.wav");
}

Registro_llamadas::~Registro_llamadas()
{
    delete ui;
}

/* Llamar al numero seleccionado */
void Registro_llamadas::llamar()
{
    // Obtenemos la fila y si hay alguna seleccionada
    int fila = this->filaSeleccionada();
    if( fila != -1)
    {
    	// Obtengo la informacion de la persona a la que queremos llamar
        QModelIndex indice = modelo->index(fila, 1, QModelIndex()),indice2 = modelo->index(fila, 0, QModelIndex());
        QVariant tel = modelo->data(indice, Qt::DisplayRole);
        QVariant nom = modelo->data(indice2, Qt::DisplayRole);
        // Genero la url y la lanzo
        url = QUrl(url_llamada + tel.toString());
        qnam.get(QNetworkRequest(url));
        // Muestro el dialog de llamada saliente con la informacion del contacto
        llam_sal->setText("Llamando a:\n" + nom.toString() + "\n" + tel.toString());
        this->addEntrada(nom.toString(),tel.toString(),false,false);
        llam_sal->show();
    }

}


/* Devuelve la fila selecionada de la tabla */
int Registro_llamadas::filaSeleccionada()
{
    // Obtenemos una lista de filas seleccionadas y devolvemos la primera
    QItemSelectionModel *selectionModel = ui->tableView->selectionModel();
    QModelIndexList indexes = selectionModel->selectedRows();
    QModelIndex index = indexes.first();
    // Devuelvo la primera fila seleccionada
    foreach (index, indexes) {
        return index.row();
    }
    return -1;
}

/* Crea las acciones que se veran al pulsar con el boton derecho sobre un contacto */
void Registro_llamadas::crearAcciones()
 {

/* Accion aadir */
    addAction = new QAction(tr("&Aadir"), this);
    addAction->setIcon(QIcon(path + "img/nuevo.png"));
    addAction->setShortcut(tr("Ctrl+N"));
    addAction->setStatusTip(tr("Crea una nueva entrada a partir de una existente"));
    connect(addAction, SIGNAL(triggered()), this, SLOT(nuevaEntrada()));

 }


/* Crea el menu que aparecera al pinchar con el boton derecho sobre las filas */
void Registro_llamadas::crearMenusContextuales()
{
        ui->tableView->addAction(addAction);
        ui->tableView->setContextMenuPolicy(Qt::ActionsContextMenu);
}


/* Muestra el dialogo de nueva entrada */
void Registro_llamadas::nuevaEntrada()
{
    int fila = this->filaSeleccionada();
    if( fila != -1)
    {
        QAbstractItemModel *modelo = ui->tableView->model();
        QString nombre = "", telefono = "";
        // Obtengo los datos
        QModelIndex indice = modelo->index(fila, 0, QModelIndex());
        nombre = modelo->data(indice, Qt::DisplayRole).toString();
        indice = modelo->index(fila, 1, QModelIndex());
        telefono = modelo->data(indice, Qt::DisplayRole).toString();
        // Muestro el dialogo con los datos
        ent->resetearEntrada(nombre,telefono);
        ent->show();
    }
}
